@@ -39,7 +39,13 @@ class UbuntuServices(services_base.ServicesBase):
         """
         Save current HAProxy totals for an expected restart.
         """
-        q = query.HAProxyQuery('/var/run/haproxy-stats.socket')
+        socket_file = '/var/run/haproxy-stats.socket'
+
+        # On a new device, the socket file won't exist.
+        if not os.path.exists(socket_file):
+            return
+
+        q = query.HAProxyQuery(socket_file)
         results = q.get_bytes_out()
 
         stats_file = cfg.CONF['worker:haproxy']['statsfile']
@@ -55,6 +61,8 @@ class UbuntuServices(services_base.ServicesBase):
 
         tcp_bo = stats_mgr.get_last_tcp_bytes()
         http_bo = stats_mgr.get_last_http_bytes()
+        unrpt_tcp_bo = stats_mgr.get_unreported_tcp_bytes()
+        unrpt_http_bo = stats_mgr.get_unreported_http_bytes()
 
         curr_tcp_bo = 0
         curr_http_bo = 0
@@ -63,6 +71,11 @@ class UbuntuServices(services_base.ServicesBase):
         if 'http' in results:
             curr_http_bo = results['http']
 
+        # If we have unreported totals, then we haven't received a METRICS
+        # call since the last restart and we need to carry over those totals.
+        curr_tcp_bo += unrpt_tcp_bo
+        curr_http_bo += unrpt_http_bo
+
         stats_mgr.save(start, end,
                        tcp_bytes=tcp_bo,
                        http_bytes=http_bo,
@@ -249,7 +262,8 @@ class UbuntuServices(services_base.ServicesBase):
         # was restarted outside of the worker's control, so we have no choice
         # but to zero the values to avoid overcharging on usage.
         if (unrpt_tcp_bo == 0 and unrpt_http_bo == 0) and \
-           (prev_tcp_bo > current_tcp_bo) or (prev_http_bo > current_http_bo):
+           ((prev_tcp_bo > current_tcp_bo) or
+               (prev_http_bo > current_http_bo)):
             LOG.warn("Forced reset of HAProxy statistics")
             prev_tcp_bo = 0
             prev_http_bo = 0
@@ -271,12 +285,14 @@ class UbuntuServices(services_base.ServicesBase):
 
         incremental_results = []
         if 'http' in results:
-            incremental_results.append(
-                ('http', unrpt_http_bo + current_http_bo - prev_http_bo)
-            )
+            value = unrpt_http_bo + current_http_bo - prev_http_bo
+            if value < 0:
+                LOG.error("Negative statistics value: %d" % value)
+            incremental_results.append(('http', value))
         if 'tcp' in results:
-            incremental_results.append(
-                ('tcp', unrpt_tcp_bo + current_tcp_bo - prev_tcp_bo)
-            )
+            value = unrpt_tcp_bo + current_tcp_bo - prev_tcp_bo
+            if value < 0:
+                LOG.error("Negative statistics value: %d" % value)
+            incremental_results.append(('tcp', value))
 
         return str(new_start), str(new_end), incremental_results
